//###########################################################################
//
// FILE:    flash_c28.c
//
//!    This example runs the Flash API usage example from flash by copying
//!    the API into RAM
//!
//    1) Build the project
//    2) Flash the .out file into the device.
//    3) Set the hardware jumpers to boot to Flash
//
//   Steps that were taken to convert the example from RAM
//    to Flash execution:
//
//    - Change the linker cmd file to reflect the flash memory map.
//    - Make sure any initialized sections are mapped to Flash.
//    - Make sure there is a branch instruction from the entry to Flash
//      at 0x13FFF0 to the beginning of code execution. This example
//      uses the DSP28x_CodeStartBranch.asm file to accomplish this.
//    - Set boot mode Jumpers to "boot to Flash"
//    - For best performance from the flash, modify the waitstates
//      and enable the flash pipeline as shown in this example.
//      Note: any code that manipulates the flash waitstate and pipeline
//      control must be run from RAM. Thus these functions are located
//      in their own memory section called ramfuncs.
//
//###########################################################################


#include "DSP28x_Project.h"     // Device Headerfile and Examples Include File

#include <string.h>

//Include Flash API example header file
#include "F28M35x_FlashAPIexample_Dual_C28x.h"

//*****************************************************************************
// FILE Flash API include file
//*****************************************************************************
#include "F021_Concerto_C28x.h"
#include "F28M35x_Ipc_drivers.h"

//Data/Program Buffer used for testing the flash API functions
#define  WORDS_IN_FLASH_BUFFER    0xFF  // Programming data buffer, words
uint16   Buffer[WORDS_IN_FLASH_BUFFER];
uint32   *Buffer32 = (uint32 *)Buffer;

// define for update APP
#define MaxSize  1024
#define IPC_Update_COM   0x00003238
unsigned int  *MarkPtr   =   (unsigned  int *)0x13F780;
#pragma DATA_SECTION(C28R_BlockBuf,"SHARERAMS0");
uint16 C28R_BlockBuf[MaxSize];

//*****************************************************************************
// Prototype of the functions used in this example
//*****************************************************************************
void Example_Error(Fapi_StatusType status);
void Example_Done(void);
void Example_CallFlashAPI(void);

//*****************************************************************************
// This is an example code demonstrating F021 Flash API usage.
// This code is in Flash
//*****************************************************************************
void main(void)
{
// Step 1. Initialize System Control:
// Enable Peripheral Clocks
// This example function is found in the F28M35x_SysCtrl.c file.
    InitSysCtrl();

//  Unlock CSM
//
//  If the API functions are going to run in unsecured RAM
//  then the CSM must be unlocked in order for the flash
//  API functions to access the flash.
//  If the flash API functions are executed from secure memory
//  then this step is not required.
    CsmUnlock();

// Step 2. Initalize GPIO:
// This example function is found in the F28M35x_Gpio.c file and
// illustrates how to set the GPIO to it's default state.
// InitGpio();  // Skipped for this example

// Step 3. Clear all interrupts and initialize PIE vector table:
// Disable CPU interrupts
    DINT;

// Initialize the PIE control registers to their default state.
// The default state is all PIE interrupts disabled and flags
// are cleared.
// This function is found in the F28M35x_PieCtrl.c file.
    InitPieCtrl();

// Disable CPU interrupts and clear all CPU interrupt flags:
    IER = 0x0000;
    IFR = 0x0000;

// Initialize the PIE vector table with pointers to the shell Interrupt
// Service Routines (ISR).
// This will populate the entire table, even if the interrupt
// is not used in this example.  This is useful for debug purposes.
// The shell ISR routines are found in DSP2802x_DefaultIsr.c.
// This function is found in F28M35x_PieVect.c.
    InitPieVectTable();

// Copy time critical code and Flash setup code to RAM
// This includes InitFlash(), Flash API functions and any functions that are
// assigned to ramfuncs section.
// The  RamfuncsLoadStart, RamfuncsLoadEnd, and RamfuncsRunStart
// symbols are created by the linker. Refer to the device .cmd file.

   memcpy(&RamfuncsRunStart, &RamfuncsLoadStart, (size_t)&RamfuncsLoadSize);

// Call Flash Initialization to setup flash waitstates
// This function must reside in RAM
    InitFlash();


    while(*MarkPtr != 0xFFFF) // update success last time, fine a unit to be FF or F1
        		{
         			MarkPtr +=4;
         		}
    if (*(MarkPtr-4) == 0xFFF1)
    {
    	while(1)
    	{
    	    if((CtoMIpcRegs.MTOCIPCSTS.all & IPC_FLAG1))
    	    {
    	      if (CtoMIpcRegs.MTOCIPCCOM == IPC_Update_COM)
    	      {
    	    	// acknowledge flag1, and flag32 acknowledge in C28 CKFA
    	    	CtoMIpcRegs.MTOCIPCACK.all |= (IPC_FLAG1);
    	    	break;
    	       }
    	    }
    	}
    }

    //  Gain pump semaphore
        FlashGainPump();
//  Jump to RAM and call the Flash API functions
    Example_CallFlashAPI();

}

//*****************************************************************************
//  Example_CallFlashAPI
//
//  This function will interface to the flash API.
//  Flash API functions used in this function are executed from RAM
//*****************************************************************************

void Example_CallFlashAPI(void)
{
    uint32 u32Index = 0;
    uint16 i = 0;
    uint16 bank_width = 0;

    Fapi_StatusType            oReturnCheck;
    Fapi_LibraryInfoType       oLibInfo;
    Fapi_FlashStatusType       oFlashStatus;
    Fapi_FlashBankSectorsType  oFlashBankSectors;
    Fapi_FlashStatusWordType   oFlashStatusWord;

    uint32  BlockEntryPoint;
    uint16  BlockLength,BlockLength1;
    uint16  Mark=0xFFF1;
    uint16  Mark1=0xFFF0;

    // Disable ECC.  ECC does not have to be disabled to do FSM operations like
    // program and erase.
    // However, on Sonata Rev. 0 silicon, due to an OTP ECC errata,
    // disable ECC to avoid ECC errors while using Flash API functions that
    // read TI-OTP
    EALLOW;
    FlashEccRegs.ECC_ENABLE.bit.ENABLE = 0x0;
    EDIS;

    EALLOW;

    // This function is required to initialize the Flash API based on System
    // frequency before any other Flash API operation can be performed
    #if CPU_FRQ_150MHZ
    oReturnCheck = Fapi_initializeAPI(F021_CPU0_BASE_ADDRESS, 150);
    #endif

    #if CPU_FRQ_100MHZ
    oReturnCheck = Fapi_initializeAPI(F021_CPU0_BASE_ADDRESS, 100);
    #endif

    #if CPU_FRQ_60MHZ
    oReturnCheck = Fapi_initializeAPI(F021_CPU0_BASE_ADDRESS, 60);
    #endif

    if(oReturnCheck |= Fapi_Status_Success)
    {
        // Check Flash API documentation for possible errors
        Example_Error(oReturnCheck);
    }
    
    // Fapi_getLibraryInfo function can be used to get the information specific
    // to the compiled version of the API library
    oLibInfo = Fapi_getLibraryInfo();

    // Before performing FSM operations, set the waitstates for FSM operations
    // calculated using RWAIT = (SYSCLK/(2*24MHz))-1
    // If RWAIT results in a fractional value, round it up to the nearest
    // integer.
    // Please note that RWAIT for read operation should be calculated
    // differently.  See Internal Memory guide section in TRM for more details.

       #if CPU_FRQ_150MHZ
       FlashCtrlRegs.FRDCNTL.bit.RWAIT = 0x3;
       #endif

       #if CPU_FRQ_100MHZ
       FlashCtrlRegs.FRDCNTL.bit.RWAIT = 0x2;
       #endif

       #if CPU_FRQ_60MHZ
       FlashCtrlRegs.FRDCNTL.bit.RWAIT = 0x1;
       #endif

    // Fapi_setActiveFlashBank function sets the Flash bank and FMC for further
    // Flash operations to be performed on the bank
    oReturnCheck = Fapi_setActiveFlashBank(Fapi_FlashBank0);
    if(oReturnCheck |= Fapi_Status_Success)
    {
        // Check Falsh API documentation for possible errors
        Example_Error(oReturnCheck);
    }

    // Fapi_getBankSectors function returns the bank starting address, number of
    // sectors, sector sizes, and bank technology type
    // Above information is returned in a structure oFlashBankSectors of type
    // Fapi_FlashBankSectorsType
    oReturnCheck = Fapi_getBankSectors(Fapi_FlashBank0,&oFlashBankSectors);
    if(oReturnCheck |= Fapi_Status_Success)
    {
        //Check Falsh API documentation for possible errors
        Example_Error(oReturnCheck);
    }
//*******************************************************************************
    if (*MarkPtr != 0xFFF1)
        {
        	u32Index = (uint32)MarkPtr;
        	oReturnCheck = Fapi_issueProgrammingCommand((uint32 *)u32Index,&Mark,
        	    					        1,
        	    			    		    0,
        	    			    		    0,
        	    			    		    Fapi_AutoEccGeneration); //Fapi_AutoEccGeneration //Fapi_DataOnly
        }
//*******************************************************************************
    // Erase Sector C
    // Sectors A have the CKFA code so leave them programmed
    oReturnCheck = Fapi_issueAsyncCommandWithAddress(Fapi_EraseSector,
                   (uint32 *)Bzero_SectorC_start);
    
    // Wait until FSM is done with erase sector operation
    while (Fapi_checkFsmForReady() != Fapi_Status_FsmReady){}
    
    // Verify that SectorL is erased.  The Erase step itself does a
    // verify as it goes.  This verify is a 2nd verification that can be done.
    oReturnCheck = Fapi_doBlankCheck((uint32 *)Bzero_SectorC_start,
                   Bzero_16KSector_u32length,
                   &oFlashStatusWord);
    
    if(oReturnCheck |= Fapi_Status_Success)
    {
        // Check Flash API documentation for possible errors
        // If Erase command fails, use Fapi_getFsmStatus() function to get the
        // FMSTAT register contents to see if any of the EV bit, ESUSP bit,
        // CSTAT bit or VOLTSTAT bit is set (Refer to API documentation for
        // more details)
        Example_Error(oReturnCheck);
    }
    
    // Erase Sector B
    // Sector C has the CKFA code so leave it programmed
    oReturnCheck = Fapi_issueAsyncCommandWithAddress(Fapi_EraseSector,
                   (uint32 *)Bzero_SectorB_start);
    
    // Wait until FSM is done with erase sector operation
    while (Fapi_checkFsmForReady() != Fapi_Status_FsmReady){}
    // Verify that SectorK is erased.  The Erase step itself does a verify as
    // it goes.  This verify is a 2nd verification that can be done.
    oReturnCheck = Fapi_doBlankCheck((uint32 *)Bzero_SectorB_start,
                   Bzero_16KSector_u32length,
                   &oFlashStatusWord);
    
    if(oReturnCheck |= Fapi_Status_Success)
    {
        // Check Flash API documentation for possible errors
        // If Erase command fails, use Fapi_getFsmStatus() function
        // to get the FMSTAT register contents
        // to see if any of the EV bit, ESUSP bit, CSTAT bit or VOLTSTAT
        // bit is set (Refer to API documentation for more details)
        Example_Error(oReturnCheck);
    }		

    // Erase Sector D
        // Sector C has the CKFA code so leave it programmed
        oReturnCheck = Fapi_issueAsyncCommandWithAddress(Fapi_EraseSector,
                       (uint32 *)Bzero_SectorD_start);

        // Wait until FSM is done with erase sector operation
        while (Fapi_checkFsmForReady() != Fapi_Status_FsmReady){}
        // Verify that SectorK is erased.  The Erase step itself does a verify as
        // it goes.  This verify is a 2nd verification that can be done.
        oReturnCheck = Fapi_doBlankCheck((uint32 *)Bzero_SectorD_start,
                       Bzero_16KSector_u32length,
                       &oFlashStatusWord);

        if(oReturnCheck |= Fapi_Status_Success)
        {
            // Check Flash API documentation for possible errors
            // If Erase command fails, use Fapi_getFsmStatus() function
            // to get the FMSTAT register contents
            // to see if any of the EV bit, ESUSP bit, CSTAT bit or VOLTSTAT
            // bit is set (Refer to API documentation for more details)
            Example_Error(oReturnCheck);
        }

        // Erase Sector E
           // Sector C has the CKFA code so leave it programmed
           oReturnCheck = Fapi_issueAsyncCommandWithAddress(Fapi_EraseSector,
                              (uint32 *)Bzero_SectorE_start);

           // Wait until FSM is done with erase sector operation
           while (Fapi_checkFsmForReady() != Fapi_Status_FsmReady){}
           // Verify that SectorK is erased.  The Erase step itself does a verify as
           // it goes.  This verify is a 2nd verification that can be done.
           oReturnCheck = Fapi_doBlankCheck((uint32 *)Bzero_SectorE_start,
                              Bzero_16KSector_u32length,
                              &oFlashStatusWord);

           if(oReturnCheck |= Fapi_Status_Success)
           {
                   // Check Flash API documentation for possible errors
                   // If Erase command fails, use Fapi_getFsmStatus() function
                   // to get the FMSTAT register contents
                   // to see if any of the EV bit, ESUSP bit, CSTAT bit or VOLTSTAT
                   // bit is set (Refer to API documentation for more details)
                   Example_Error(oReturnCheck);
            }
       // Erase Sector F
           oReturnCheck = Fapi_issueAsyncCommandWithAddress(Fapi_EraseSector,
                          (uint32 *)Bzero_SectorF_start);

           // Wait until FSM is done with erase sector operation
           while (Fapi_checkFsmForReady() != Fapi_Status_FsmReady){}

           // Verify that SectorL is erased.  The Erase step itself does a
           // verify as it goes.  This verify is a 2nd verification that can be done.
           oReturnCheck = Fapi_doBlankCheck((uint32 *)Bzero_SectorF_start,
                          Bzero_16KSector_u32length,
                          &oFlashStatusWord);

           if(oReturnCheck |= Fapi_Status_Success)
           {
               // Check Flash API documentation for possible errors
               // If Erase command fails, use Fapi_getFsmStatus() function to get the
               // FMSTAT register contents to see if any of the EV bit, ESUSP bit,
               // CSTAT bit or VOLTSTAT bit is set (Refer to API documentation for
               // more details)
               Example_Error(oReturnCheck);
           }
// 28 erase end
//******************************************************************************
    // acknowledge to m3 indicate that 28 is ready for receive data and program
    // flag1 is acknowledged in 28 main
        CtoMIpcRegs.MTOCIPCACK.all |= (IPC_FLAG32);
    // communication with m3 by IPC to receive data and program
    while(1)
    {
    	while (!(CtoMIpcRegs.MTOCIPCSTS.all & IPC_FLAG1)){}
    	if (CtoMIpcRegs.MTOCIPCCOM == IPC_DATA_WRITE_16)
    	{
    		BlockEntryPoint = CtoMIpcRegs.MTOCIPCADDR;
    		if(BlockEntryPoint==0x55555555)
    		{
    		   		 //TODO clear update mark save in address 0x3E8000 ::clear the mark
    			if (*MarkPtr == 0xFFF1)
    			{
    			  u32Index = (uint32)MarkPtr+4;
    			  oReturnCheck = Fapi_issueProgrammingCommand((uint32 *)u32Index,&Mark1,
    			    		     1,
    			    		     0,
    			    		     0,
    			    		     Fapi_AutoEccGeneration); //Fapi_AutoEccGeneration //Fapi_DataOnly
    			 }
    			// FlashLeavePump();
    		 break;
    		 }
    		BlockLength = (uint16)CtoMIpcRegs.MTOCIPCDATAW;
    		BlockLength1 = BlockLength;
    		// begin program block data from shared RAM s0
    		for(i=0, u32Index = BlockEntryPoint;
    		       (u32Index < (BlockEntryPoint + BlockLength))
    		       && (oReturnCheck == Fapi_Status_Success); )
    		    {
    			    if(BlockLength1 < 8)
    			    {
    		           oReturnCheck = Fapi_issueProgrammingCommand((uint32 *)u32Index,C28R_BlockBuf+i,
    		        		          BlockLength1,
    		                          0,
    		                          0,
    		                          Fapi_AutoEccGeneration);
    		           u32Index +=BlockLength1;
    		           i+=BlockLength1;

    			    }
    			    else
    			    {

    			      bank_width = u32Index & 0x0000000F;
    			       if(bank_width >= 8) // FIXME
    			       {
    			    	  oReturnCheck = Fapi_issueProgrammingCommand((uint32 *)u32Index,C28R_BlockBuf+i,
    			    	    			 16-bank_width,
    			    	    			 0,
    			    	    			 0,
    			    	    			 Fapi_AutoEccGeneration);//Fapi_AutoEccGeneration
    			    	  while(Fapi_checkFsmForReady() == Fapi_Status_FsmBusy);
    			    	  if(oReturnCheck |= Fapi_Status_Success)
    			    	  {
    			    	    // Check Flash API documentation for possible errors
    			    	    Example_Error(oReturnCheck);
    			    	   }
    			    	  u32Index += (16-bank_width);
    			          BlockLength1 -= (16-bank_width);
    			    	  i += (16-bank_width);
    			    	 }
    			       else if(bank_width <8) // FIXME
    			           			       {
    			           			    	  oReturnCheck = Fapi_issueProgrammingCommand((uint32 *)u32Index,C28R_BlockBuf+i,
    			           			    	    			 8-bank_width,
    			           			    	    			 0,
    			           			    	    			 0,
    			           			    	    			 Fapi_AutoEccGeneration);//Fapi_AutoEccGeneration

    			           			    	  while(Fapi_checkFsmForReady() == Fapi_Status_FsmBusy);
    			           			    	  if(oReturnCheck |= Fapi_Status_Success)
    			           			    	  {
    			           			    	    // Check Flash API documentation for possible errors
    			           			    	    Example_Error(oReturnCheck);
    			           			    	   }
    			           			    	 u32Index += (8-bank_width);
    			           			          BlockLength1 -= (8-bank_width);
    			           			    	  i += (8-bank_width);

    			           			    	 }
    			     /*   else
    			       {
    			       oReturnCheck = Fapi_issueProgrammingCommand((uint32 *)u32Index,C28R_BlockBuf+i,
    			    	    		  8,
    			    	    		  0,
    			    	    		  0,
    			    	              Fapi_AutoEccGeneration);//Fapi_AutoEccGeneration
    			       BlockLength1 -= 8;
    			       i+= 8;
    			       }*/
    			    }
/*
    		        while(Fapi_checkFsmForReady() == Fapi_Status_FsmBusy);

    		        if(oReturnCheck |= Fapi_Status_Success)
    		        {
    		        // Check Flash API documentation for possible errors
    		        Example_Error(oReturnCheck);
    		        }

    		        // Read FMSTAT register contents to know the status of FSM after
    		        // program command for any debug
    		        oFlashStatus = Fapi_getFsmStatus();
*/

    		    }// program for() end

            // acknowledge after 28 finish block program
    		// indicate M3 send "ok" to pc for next block date receive
    	    CtoMIpcRegs.MTOCIPCACK.all |= (IPC_FLAG1 | IPC_FLAG32);
    	}// if MTOCIPCCOM end
    }// while(1) end
    EDIS;// line 159 EALLOW
    FlashLeavePump();
}//Example_CallFlashAPI() end


//******************************************************************************
// For this example, if an error is found just stop here
//******************************************************************************
#pragma CODE_SECTION(Example_Error,"ramfuncs");
void Example_Error(Fapi_StatusType status)
{
    //  Error code will be in the status parameter
        asm("    ESTOP0");
}


//******************************************************************************
//  For this example, once we are done just stop here
//******************************************************************************
#pragma CODE_SECTION(Example_Done,"ramfuncs");
void Example_Done(void)
{
    asm("    ESTOP0");
}

